@@ -2,7 +2,7 @@
 // Syd: rock-solid application kernel
 // benches/sys/getpid.rs: getpid microbenchmarks
 //
-// Copyright (c) 2024 Ali Polatel <alip@chesswob.org>
+// Copyright (c) 2024, 2025 Ali Polatel <alip@chesswob.org>
 // Based in part upon gVisor's getpid_benchmark.cc which is:
 //   Copyright 2020 The gVisor Authors.
 //   SPDX-License-Identifier: Apache-2.0
@@ -14,15 +14,13 @@
 // invoke the kernel via a simple `syscall!` macro.
 
 use brunch::{benches, Bench};
-use libc::SYS_getpid;
-use nix::errno::Errno;
-use syd::syscall;
+use libc::{syscall, SYS_getpid};
 
 /// Benchmark calling the `SYS_getpid` syscall via our macro.
 fn bm_getpid() {
     // Just call `syscall!` in a tight loop.
-    // getpid(2) never returns error.
-    let _ = syscall!(SYS_getpid);
+    // SAFETY: getpid(2) never returns error.
+    let _ = unsafe { syscall(SYS_getpid) };
 }
 
 fn main() {
