@@ -17,8 +17,8 @@ umask 077
 # Disable coredumps.
 ulimit -c 0
 
-SYD="${CARGO_BIN_EXE_syd:-syd}"
 PERF="${PERF:-perf}"
+SYD="${CARGO_BIN_EXE_syd:-syd}"
 PROF="${SYD_PERF_PROFILE:-paludis}"
 
 DIR="$(mktemp -d --tmpdir=/tmp syd-lnx.XXXXXXXXXX)"
@@ -37,4 +37,4 @@ export SYD_PID_FN=/tmp/syd.pid
 "${PERF}" "${@}" -- \
     "${SYD}" -q -p"${PROF}" -pP -m 'allow/all+/***' -mlock:on \
         -- \
-        sh -cex 'make defconfig && make -j && make clean'
+        sh -cex 'make defconfig && make -j$(nproc) && make clean'
