@@ -15,8 +15,12 @@ ulimit -c unlimited
 # Force TTY output.
 export SYD_FORCE_TTY=YesPlease
 
-# Timeout is 20 minutes per-test,
-# unless otherwise specified.
+# Enable path hiding and ioctl sandboxing for wider coverage.
+# This works because NIX profile includes the PALUDIS profile.
+export SYD_PALUDIS_LPATH=1
+export SYD_PALUDIS_IOCTL=1
+
+# Timeout is 20 minutes per-test unless otherwise specified.
 SYD_TEST_TIMEOUT=${SYD_TEST_TIMEOUT:-20m}
 
 export SYD_LOG=${SYD_LOG:-notice}
