@@ -37,6 +37,9 @@ use serde::{ser::SerializeMap, Serialize, Serializer};
 
 use crate::{config::*, lookup::FileType, XPath};
 
+pub(crate) const UIO_MAXIOV: usize = libc::UIO_MAXIOV as usize;
+pub(crate) const UIO_MAXLEN: usize = 256_000;
+
 // x32 compatibility
 // See https://sourceware.org/bugzilla/show_bug.cgi?id=16437
 #[cfg(all(target_arch = "x86_64", target_pointer_width = "32"))]
