@@ -271,7 +271,7 @@ pub(crate) fn safe_openat2<Fd: AsFd, P: NixPath + ?Sized>(
             dirfd.as_fd().as_raw_fd(),
             cstr.as_ptr(),
             std::ptr::addr_of_mut!(how),
-            std::mem::size_of::<libc::open_how>(),
+            size_of::<libc::open_how>(),
             *OPENAT2_COOKIE_ARG4,
             *OPENAT2_COOKIE_ARG5,
         )
