@@ -17,8 +17,8 @@ use nix::{
 };
 
 use crate::{
-    fs::FsFlags,
     kernel::{syscall_path_handler, to_atflags},
+    lookup::FsFlags,
     req::{PathArgs, SysArg, SysFlags, UNotifyEventRequest},
 };
 
