@@ -10,8 +10,9 @@ use libseccomp::ScmpNotifResp;
 use nix::{errno::Errno, fcntl::AtFlags};
 
 use crate::{
-    fs::{is_executable, FileType, FsFlags, AT_EXECVE_CHECK},
+    fs::{is_executable, AT_EXECVE_CHECK},
     kernel::{sandbox_path, to_atflags},
+    lookup::{FileType, FsFlags},
     req::{SysArg, SysFlags, UNotifyEventRequest},
     sandbox::Capability,
 };
