@@ -12,7 +12,7 @@ use libseccomp::ScmpNotifResp;
 use nix::errno::Errno;
 
 use crate::{
-    fs::{CanonicalPath, FileInfo},
+    lookup::{CanonicalPath, FileInfo},
     req::UNotifyEventRequest,
     sandbox::Capability,
 };
