@@ -282,17 +282,18 @@ fn syscall_mem_handler(
             // 2. `path` is a remote-fd transfer which asserts `path.dir` is Some.
             #[expect(clippy::disallowed_methods)]
             let fd = path.dir.take().unwrap();
-            let pfd = XPathBuf::from_self_fd(fd.as_raw_fd());
 
-            let mut file =
-                match safe_open_msym(PROC_FILE(), &pfd, OFlag::O_RDONLY, ResolveFlag::empty())
-                    .map(File::from)
-                {
-                    Ok(file) => file,
-                    Err(_) => {
-                        return request.fail_syscall(Errno::EBADF);
-                    }
-                };
+            let mut file = match XPathBuf::from_self_fd(fd.as_raw_fd())
+                .and_then(|pfd| {
+                    safe_open_msym(PROC_FILE(), &pfd, OFlag::O_RDONLY, ResolveFlag::empty())
+                })
+                .map(File::from)
+            {
+                Ok(file) => file,
+                Err(_) => {
+                    return request.fail_syscall(Errno::EBADF);
+                }
+            };
 
             if restrict_exec_stack {
                 // Step 4: Check for non-executable stack.
