@@ -150,8 +150,10 @@ use nix::{
 };
 
 use crate::{
-    fs::{fd_status_flags, to_valid_fd, CanonicalPath, FileInfo, FileType},
+    fs::{fd_status_flags, to_valid_fd},
     ioctl::{ioctl_names_get, Ioctl},
+    log::log_is_main,
+    lookup::{CanonicalPath, FileInfo, FileType},
     notice,
     path::{XPath, XPathBuf},
     req::{PathArgs, SysArg, UNotifyEventRequest},
@@ -337,6 +339,8 @@ pub(crate) fn sandbox_path(
             }
         }
         Action::Deny | Action::Filter => Err(deny_errno),
+        //Do NOT panic the main thread!
+        Action::Panic if log_is_main(std::thread::current().id()) => Err(deny_errno),
         Action::Panic => panic!(),
         Action::Exit => std::process::exit(deny_errno as i32),
         Action::Stop => {
