@@ -86,13 +86,12 @@ pub(crate) fn sys_socketcall(request: UNotifyEventRequest) -> ScmpNotifResp {
     let mut args = [0u64; ARGLEN];
     #[expect(clippy::arithmetic_side_effects)]
     let bufsiz = sizeof_ulong * ARGLEN;
-    let mut buf = Vec::new();
-    if buf.try_reserve(bufsiz).is_err() {
-        return request.fail_syscall(Errno::ENOMEM);
-    }
-    buf.resize(bufsiz, 0);
-    match request.read_mem(&mut buf, req.data.args[1]) {
-        Ok(n) if n == bufsiz => {
+    match request.read_vec(req.data.args[1], bufsiz) {
+        Ok(buf) if buf.len() != bufsiz => {
+            // Short read.
+            return request.fail_syscall(Errno::EFAULT);
+        }
+        Ok(buf) => {
             for (i, chunk) in buf.chunks_exact(sizeof_ulong).enumerate() {
                 match sizeof_ulong {
                     4 => match chunk.try_into() {
@@ -111,10 +110,7 @@ pub(crate) fn sys_socketcall(request: UNotifyEventRequest) -> ScmpNotifResp {
                 }
             }
         }
-        _ => {
-            // Short read or error.
-            return request.fail_syscall(Errno::EFAULT);
-        }
+        Err(errno) => return request.fail_syscall(errno),
     }
 
     syscall_network_handler(request, &args, op)
@@ -396,12 +392,8 @@ fn get_addr(
     #[expect(clippy::arithmetic_side_effects)]
     let addr_len = addr_len.min(size_of::<libc::sockaddr_un>() + UNIX_PATH_MAX);
 
-    let mut addr = Vec::new();
-    addr.try_reserve(addr_len).or(Err(Errno::ENOMEM))?;
-    addr.resize(addr_len, 0);
-    let addr_len = addr_len.try_into().or(Err(Errno::EINVAL))?;
-
-    request.read_mem(&mut addr, addr_remote)?;
+    let addr = request.read_vec(addr_remote, addr_len)?;
+    let addr_len = addr.len().try_into().or(Err(Errno::EINVAL))?;
     let addr = addr.as_ptr().cast();
 
     // SAFETY: Invoking `SockaddrStorage::from_raw` is safe because:
