@@ -100,7 +100,7 @@ pub(crate) fn handle_recvfrom(
     let addrlen = if args[5] != 0 {
         const SIZEOF_SOCKLEN_T: usize = size_of::<libc::socklen_t>();
         let mut buf = [0u8; SIZEOF_SOCKLEN_T];
-        if request.read_mem(&mut buf, args[5])? == SIZEOF_SOCKLEN_T {
+        if request.read_mem(&mut buf, args[5], SIZEOF_SOCKLEN_T)? == SIZEOF_SOCKLEN_T {
             // libc defines socklen_t as u32,
             // however we should check for negative values
             // and return EINVAL as necessary.
