@@ -21,7 +21,7 @@ use zeroize::Zeroizing;
 use crate::{
     compat::{
         addr_family, cmsg_len_32, cmsg_space_32, msghdr, msghdr32, MsgFlags, PF_ALG, PF_INET,
-        PF_INET6, PF_NETLINK, PF_UNIX,
+        PF_INET6, PF_NETLINK, PF_UNIX, UIO_MAXIOV, UIO_MAXLEN,
     },
     confine::scmp_arch_bits,
     fs::{get_nonblock, has_recv_timeout, is_same_vm},
@@ -61,10 +61,7 @@ pub(crate) fn handle_sendmsg(
     } else {
         size_of::<msghdr>()
     };
-    let mut buf = Zeroizing::new(Vec::new());
-    buf.try_reserve(size).or(Err(Errno::ENOMEM))?;
-    buf.resize(size, 0);
-    request.read_mem(&mut buf, args[1])?;
+    let buf = request.read_vec_zeroed(args[1], size)?;
 
     let msg = if is32 {
         // SAFETY: See below.
@@ -127,11 +124,8 @@ pub(crate) fn handle_sendmsg(
         #[expect(clippy::useless_conversion)]
         let cmsg_len = usize::try_from(msg.msg_controllen)
             .or(Err(Errno::EINVAL))?
-            .min(1000000); // SAFETY: Cap at 1mio.
-        let mut cmsg_buf = Vec::new();
-        cmsg_buf.try_reserve(cmsg_len).or(Err(Errno::ENOMEM))?;
-        cmsg_buf.resize(cmsg_len, 0);
-        request.read_mem(&mut cmsg_buf, msg.msg_control as u64)?;
+            .min(UIO_MAXLEN); // SAFETY: Cap at UIO_MAXLEN.
+        let cmsg_buf = request.read_vec(msg.msg_control as u64, cmsg_len)?;
         Some(parse_control_messages(request, &cmsg_buf)?)
     } else {
         None
@@ -246,28 +240,20 @@ pub(crate) fn handle_sendmsg(
         // SAFETY: The msg_iovlen member of the msghdr struct
         // must not be fully trusted, it can be overly large,
         // and allocating a Vector of that capacity may overflow.
+        // Here we cap it to UIO_MAXIOV for safety.
         #[expect(clippy::useless_conversion)]
         let len = usize::try_from(msg.msg_iovlen)
-            .or(Err(Errno::EINVAL))?
-            .min(1000000); // Cap count at 1mio
+            .or(Err(Errno::EMSGSIZE))?
+            .min(UIO_MAXIOV);
+
         let size = if is32 {
             len.checked_mul(size_of::<crate::compat::iovec32>())
         } else {
             len.checked_mul(size_of::<libc::iovec>())
         }
-        .ok_or(Errno::EINVAL)?;
-        let mut buf = Zeroizing::new(Vec::new());
-        buf.try_reserve(size).or(Err(Errno::ENOMEM))?;
-        buf.resize(size, 0);
-        request.read_mem(&mut buf, msg.msg_iov as u64)?;
+        .ok_or(Errno::EMSGSIZE)?;
 
-        // SAFETY: This operation assumes that the buffer (`buf`) contains a valid sequence of bytes
-        // that correctly represent an array of `iovec` structures. This is ensured by the preceding
-        // code that reads memory into `buf` with proper length calculation. The length `len` is
-        // derived from `msg.msg_iovlen` and capped to prevent overflow, ensuring that we do not
-        // exceed the allocation size of `buf`. The conversion to a pointer and then to a slice
-        // of `iovec` is safe under these conditions, assuming the memory layout of `iovec` is
-        // correct and `buf` is correctly sized and aligned.
+        let buf = request.read_vec_zeroed(msg.msg_iov as u64, size)?;
         let mut iovecs: Vec<libc::iovec> = Vec::new();
         if is32 {
             for chunk in buf.chunks(size_of::<crate::compat::iovec32>()) {
@@ -285,25 +271,19 @@ pub(crate) fn handle_sendmsg(
 
         for iov in iovecs {
             if iov.iov_base.is_null() || iov.iov_len == 0 {
-                // XXX: This happens with socketcall on x86, why?
                 continue;
             }
 
-            // Cap the length to a maximum value to avoid large allocations.
-            // SAFETY: The maximum length cap prevents excessive memory
-            // allocation based on untrusted `iov_len`.
-            let iov_len = iov.iov_len.min(1000000); // Cap count at 1mio
-
-            // Allocate a buffer to read into. This buffer size is now capped.
-            let mut data_buf = Vec::new();
-            data_buf.try_reserve(iov_len).or(Err(Errno::ENOMEM))?;
-            data_buf.resize(iov_len, 0);
+            // Validate length to avoid large allocations.
+            #[expect(clippy::useless_conversion)]
+            let iov_len = usize::try_from(iov.iov_len).or(Err(Errno::EMSGSIZE))?;
+            if iov_len > UIO_MAXLEN {
+                // Invalid iovec count: Too large.
+                return Err(Errno::EMSGSIZE);
+            }
 
             // Read the memory from the remote process into our buffer.
-            // SAFETY: This operation relies on the correctness of
-            // `iov_base` as a pointer into the remote process's memory and
-            // the capped `iov_len`.
-            request.read_mem(&mut data_buf, iov.iov_base as u64)?;
+            let data_buf = request.read_vec(iov.iov_base as u64, iov_len)?;
 
             // Keep the pointer accessible, IoSlice needs a valid reference.
             io_buffers.try_reserve(1).or(Err(Errno::ENOMEM))?;
@@ -405,18 +385,16 @@ pub(crate) fn handle_sendmmsg(
     }
     .ok_or(Errno::EINVAL)?;
 
-    // Read mmsghdr structures from remote process memory
-    let mut buf = Zeroizing::new(Vec::new());
-    buf.try_reserve(size).or(Err(Errno::ENOMEM))?;
-    buf.resize(size, 0);
-    request.read_mem(&mut buf, addr)?;
+    // Read mmsghdr structures from remote process memory.
+    let buf = request.read_vec_zeroed(addr, size)?;
 
     let mut mmsghdrs: Vec<crate::compat::mmsghdr> = Vec::new();
     if is32 {
         for chunk in buf.chunks(size_of::<crate::compat::mmsghdr32>()) {
             // SAFETY: See the relevant comment in handle_sendmsg.
-            let mmsghdr: crate::compat::mmsghdr32 =
-                unsafe { std::ptr::read_unaligned(chunk.as_ptr() as *const _) };
+            let mmsghdr: crate::compat::mmsghdr32 = unsafe {
+                std::ptr::read_unaligned(chunk.as_ptr() as *const crate::compat::mmsghdr32)
+            };
             mmsghdrs.try_reserve(1).or(Err(Errno::ENOMEM))?;
             mmsghdrs.push(mmsghdr.into());
         }
@@ -424,7 +402,9 @@ pub(crate) fn handle_sendmmsg(
         for chunk in buf.chunks(size_of::<crate::compat::mmsghdr>()) {
             mmsghdrs.try_reserve(1).or(Err(Errno::ENOMEM))?;
             // SAFETY: See the relevant comment in handle_sendmsg.
-            mmsghdrs.push(unsafe { std::ptr::read_unaligned(chunk.as_ptr() as *const _) });
+            mmsghdrs.push(unsafe {
+                std::ptr::read_unaligned(chunk.as_ptr() as *const crate::compat::mmsghdr)
+            });
         }
     };
 
@@ -485,11 +465,8 @@ pub(crate) fn handle_sendmmsg(
             #[expect(clippy::useless_conversion)]
             let cmsg_len = usize::try_from(msg.msg_controllen)
                 .or(Err(Errno::EINVAL))?
-                .min(1000000); // SAFETY: Cap at 1mio.
-            let mut cmsg_buf = Vec::new();
-            cmsg_buf.try_reserve(cmsg_len).or(Err(Errno::ENOMEM))?;
-            cmsg_buf.resize(cmsg_len, 0);
-            request.read_mem(&mut cmsg_buf, msg.msg_control as u64)?;
+                .min(UIO_MAXLEN); // SAFETY: Cap at UIO_MAXLEN.
+            let cmsg_buf = request.read_vec(msg.msg_control as u64, cmsg_len)?;
             Some(parse_control_messages(request, &cmsg_buf)?)
         } else {
             None
@@ -610,21 +587,20 @@ pub(crate) fn handle_sendmmsg(
             // SAFETY: The msg_iovlen member of the msghdr struct
             // must not be fully trusted, it can be overly large,
             // and allocating a Vector of that capacity may overflow.
+            // Here we cap it to UIO_MAXIOV for safety.
             #[expect(clippy::useless_conversion)]
             let len = usize::try_from(msg.msg_iovlen)
-                .or(Err(Errno::EINVAL))?
-                .min(1000000); // Cap count at 1mio.
+                .or(Err(Errno::EMSGSIZE))?
+                .min(UIO_MAXIOV);
+
             let size = if is32 {
                 len.checked_mul(size_of::<crate::compat::iovec32>())
             } else {
                 len.checked_mul(size_of::<libc::iovec>())
             }
-            .ok_or(Errno::EINVAL)?;
-            let mut buf = Zeroizing::new(Vec::new());
-            buf.try_reserve(size).or(Err(Errno::ENOMEM))?;
-            buf.resize(size, 0);
-            request.read_mem(&mut buf, msg.msg_iov as u64)?;
+            .ok_or(Errno::EMSGSIZE)?;
 
+            let buf = request.read_vec_zeroed(msg.msg_iov as u64, size)?;
             let mut iovecs: Vec<libc::iovec> = Vec::new();
             if is32 {
                 for chunk in buf.chunks(size_of::<crate::compat::iovec32>()) {
@@ -643,23 +619,23 @@ pub(crate) fn handle_sendmmsg(
             };
 
             for iov in iovecs {
-                // Cap the length to a maximum value to avoid large allocations.
-                // SAFETY: The maximum length cap prevents excessive memory
-                // allocation based on untrusted `iov_len`.
-                let iov_len = iov.iov_len.min(1000000); // Cap count at 1mio
+                if iov.iov_base.is_null() || iov.iov_len == 0 {
+                    continue;
+                }
 
-                // Allocate a buffer to read into. This buffer size is now capped.
-                let mut data_buf = Vec::new();
-                data_buf.try_reserve(iov_len).or(Err(Errno::ENOMEM))?;
-                data_buf.resize(iov_len, 0);
+                // Validate length to avoid large allocations.
+                #[expect(clippy::useless_conversion)]
+                let iov_len = usize::try_from(iov.iov_len).or(Err(Errno::EMSGSIZE))?;
+                if iov_len > UIO_MAXLEN {
+                    // Invalid iovec count: Too large.
+                    return Err(Errno::EMSGSIZE);
+                }
 
                 // Read the memory from the remote process into our buffer.
-                // SAFETY: This operation relies on the correctness of
-                // `iov_base` as a pointer into the remote process's memory and
-                // the capped `iov_len`.
-                request.read_mem(&mut data_buf, iov.iov_base as u64)?;
+                let data_buf = request.read_vec(iov.iov_base as u64, iov_len)?;
 
                 // Keep the pointer accessible, IoSlice needs a valid reference.
+                io_buffers.try_reserve(1).or(Err(Errno::ENOMEM))?;
                 io_buffers.push(data_buf);
             }
             for buffer in &io_buffers {
