@@ -944,7 +944,7 @@ fn handle_crypt(
             let mut flags = safe_flags;
             flags.remove(OFlag::O_NOFOLLOW);
 
-            let pfd = XPathBuf::from_self_fd(crypt_fd);
+            let pfd = XPathBuf::from_self_fd(crypt_fd)?;
             match safe_open_msym(PROC_FILE(), &pfd, flags, ResolveFlag::empty()) {
                 Ok(fd) if retry_on_eintr(|| lock_fd(&fd, false, false)).is_ok() => {
                     // Return shared fd.
@@ -1038,7 +1038,7 @@ fn handle_crypt(
 
     // Reopen a new instance to pass to the sandbox process.
     // This instance points to a different open file description!
-    let pfd = XPathBuf::from_self_fd(fd.as_raw_fd());
+    let pfd = XPathBuf::from_self_fd(fd.as_raw_fd())?;
 
     let mut flags = safe_flags;
     flags.remove(
@@ -1172,7 +1172,7 @@ fn do_open_crypt(path: &CanonicalPath, mut safe_flags: OFlag) -> Result<OwnedFd,
     // SAFETY: ^^ empty base asserts dir is Some.
     #[expect(clippy::disallowed_methods)]
     let fd = path.dir.as_ref().map(|fd| fd.as_raw_fd()).unwrap();
-    let pfd = XPathBuf::from_self_fd(fd);
+    let pfd = XPathBuf::from_self_fd(fd)?;
 
     // Reopen the `O_PATH` path fd with the requested flags.
     safe_open_msym(PROC_FILE(), &pfd, safe_flags, ResolveFlag::empty())
@@ -1194,7 +1194,7 @@ fn do_open(
     // SAFETY: ^^ empty base asserts dir is Some.
     #[expect(clippy::disallowed_methods)]
     let fd = path.dir.as_ref().map(|fd| fd.as_raw_fd()).unwrap();
-    let pfd = XPathBuf::from_self_fd(fd);
+    let pfd = XPathBuf::from_self_fd(fd)?;
 
     // Reopen the `O_PATH` path fd with the requested flags.
     flags.remove(OFlag::O_CREAT);
