@@ -32,7 +32,7 @@ pub(crate) fn sys_prctl(request: UNotifyEventRequest) -> ScmpNotifResp {
     drop(sandbox); // release the read-lock.
 
     let mut buf = [0u8; 15];
-    let name = match request.read_mem(&mut buf, ptr) {
+    let name = match request.read_mem(&mut buf, ptr, 15) {
         Ok(len) => {
             let nil = memchr(0, &buf[..len]).unwrap_or(len);
             &buf[..nil]
