@@ -100,8 +100,7 @@ pub(crate) fn sysexit_chdir(
     };
 
     // SAFETY: Validate /proc/$pid/cwd against TOCTTOU!
-    let mut pfd = XPathBuf::from_pid(pid);
-    pfd.push(b"cwd");
+    let pfd = XPathBuf::from_cwd(pid)?;
     let stx_cwd = match statx(PROC_FILE(), &pfd, 0, FileInfo::mask()) {
         Ok(stx) => stx,
         Err(errno) => {
