@@ -224,7 +224,7 @@ pub(crate) fn sysenter_exec(
     );
     let mut file = match path.dir.take() {
         Some(MaybeFd::Owned(fd)) => {
-            let pfd = XPathBuf::from_self_fd(fd.as_raw_fd());
+            let pfd = XPathBuf::from_self_fd(fd.as_raw_fd())?;
             safe_open_msym(PROC_FILE(), &pfd, OFlag::O_RDONLY, ResolveFlag::empty())
                 .map(File::from)?
         }
