@@ -163,7 +163,7 @@ pub(crate) fn handle_ptrace_sysenter(
             let ip_mem = if ip >= *MMAP_MIN_ADDR {
                 let mut ip_mem = [0u8; 64];
                 // SAFETY: This is a ptrace hook, the PID cannot be validated.
-                match unsafe { process.read_mem(&mut ip_mem, ip) } {
+                match unsafe { process.read_mem(&mut ip_mem, ip, 64) } {
                     Ok(_) => Some(ip_mem),
                     Err(Errno::EFAULT) => None,
                     Err(Errno::ESRCH) => return Err(Errno::ESRCH),
@@ -181,7 +181,7 @@ pub(crate) fn handle_ptrace_sysenter(
             let sp_mem = if sp >= *MMAP_MIN_ADDR {
                 let mut sp_mem = [0u8; 64];
                 // SAFETY: This is a ptrace hook, the PID cannot be validated.
-                match unsafe { process.read_mem(&mut sp_mem, sp) } {
+                match unsafe { process.read_mem(&mut sp_mem, sp, 64) } {
                     Ok(_) => Some(sp_mem),
                     Err(Errno::EFAULT) => None,
                     Err(Errno::ESRCH) => return Err(Errno::ESRCH),
@@ -259,11 +259,11 @@ pub(crate) fn handle_ptrace_sysenter(
 
             if let Some(mut ip_mem) = ip_mem {
                 // SAFETY: This is a ptrace hook, the PID cannot be validated.
-                let _ = unsafe { process.read_mem(&mut ip_mem, ip) };
+                let _ = unsafe { process.read_mem(&mut ip_mem, ip, 64) };
             }
             if let Some(mut sp_mem) = sp_mem {
                 // SAFETY: ditto.
-                let _ = unsafe { process.read_mem(&mut sp_mem, sp) };
+                let _ = unsafe { process.read_mem(&mut sp_mem, sp, 64) };
             }
 
             // Terminate the process.
