@@ -12,8 +12,8 @@ use libseccomp::ScmpNotifResp;
 use nix::{errno::Errno, unistd::symlinkat, NixPath};
 
 use crate::{
-    fs::FsFlags,
     kernel::sandbox_path,
+    lookup::FsFlags,
     req::{RemoteProcess, SysArg, UNotifyEventRequest},
     sandbox::Capability,
 };
