@@ -12,7 +12,7 @@
 use libseccomp::ScmpNotifResp;
 use nix::errno::Errno;
 
-use crate::{config::MMAP_MIN_ADDR, hook::UNotifyEventRequest, sysinfo::SysInfo};
+use crate::{config::MMAP_MIN_ADDR, req::UNotifyEventRequest, sysinfo::SysInfo};
 
 pub(crate) fn sys_sysinfo(request: UNotifyEventRequest) -> ScmpNotifResp {
     syscall_handler!(request, |request: UNotifyEventRequest| {
