@@ -21,11 +21,12 @@ use crate::{
         XATTR_SIZE_MAX,
     },
     config::ROOT_FILE,
-    fs::{denyxattr, filterxattr, FsFlags},
     kernel::{syscall_path_handler, to_atflags},
+    lookup::FsFlags,
     path::XPath,
     req::{PathArgs, SysArg, SysFlags, UNotifyEventRequest},
     sandbox::SandboxGuard,
+    xattr::{denyxattr, filterxattr},
 };
 
 pub(crate) fn sys_getxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
@@ -99,9 +100,8 @@ pub(crate) fn sys_fgetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
         // and allocating a Vector of that capacity may overflow.
         let len = to_len_cap(req.data.args[3], XATTR_SIZE_MAX)?;
         let mut buf = if len > 0 {
-            let mut buf = Vec::new();
+            let mut buf: Vec<u8> = Vec::new();
             buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
-            buf.resize(len, 0);
             Some(buf)
         } else {
             None
@@ -113,7 +113,7 @@ pub(crate) fn sys_fgetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
 
         #[expect(clippy::cast_sign_loss)]
         // SAFETY: In libc we trust.
-        let n = match Errno::result(unsafe {
+        let mut n = match Errno::result(unsafe {
             libc::fgetxattr(fd.as_raw_fd(), name, ptr.cast(), len)
         }) {
             Ok(n) => n as usize,
@@ -126,8 +126,12 @@ pub(crate) fn sys_fgetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
             Err(errno) => return Err(errno),
         };
 
-        if let Some(buf) = buf {
-            request.write_mem(&buf[..n], req.data.args[2])?;
+        if let Some(mut buf) = buf {
+            // SAFETY: n was returned by the host Linux kernel.
+            unsafe { buf.set_len(n) };
+            buf.shrink_to_fit();
+
+            n = request.write_mem(&buf, req.data.args[2])?;
         }
 
         #[expect(clippy::cast_possible_wrap)]
@@ -249,12 +253,9 @@ pub(crate) fn sys_lsetxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
             (None, 0)
         } else {
             let len = to_len_val(req.data.args[3], XATTR_SIZE_MAX)?;
-            let mut buf = Vec::new();
-            buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
-            buf.resize(len, 0);
-            let n = request.read_mem(&mut buf, req.data.args[2])?;
-            buf.truncate(n);
-            (Some(buf), n)
+            let buf = request.read_vec(req.data.args[2], len)?;
+            let len = buf.len();
+            (Some(buf), len)
         };
         let buf = buf.as_ref().map_or(std::ptr::null(), |b| b.as_ptr()) as *const libc::c_void;
 
@@ -350,7 +351,6 @@ pub(crate) fn sys_flistxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
             let mut buf = if len > 0 {
                 let mut buf = Vec::new();
                 buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
-                buf.resize(len, 0);
                 Some(buf)
             } else {
                 None
@@ -362,7 +362,7 @@ pub(crate) fn sys_flistxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
 
             #[expect(clippy::cast_sign_loss)]
             // SAFETY: In libc we trust.
-            let n = match Errno::result(unsafe { libc::flistxattr(fd.as_raw_fd(), ptr, len) }) {
+            let mut n = match Errno::result(unsafe { libc::flistxattr(fd.as_raw_fd(), ptr, len) }) {
                 Ok(n) => n as usize,
                 Err(Errno::ERANGE) if len == XATTR_LIST_MAX => {
                     // SAFETY: Avoid a well-behaving process from
@@ -373,20 +373,22 @@ pub(crate) fn sys_flistxattr(request: UNotifyEventRequest) -> ScmpNotifResp {
                 Err(errno) => return Err(errno),
             };
 
-            let n = if let Some(buf) = buf {
-                // SAFETY: Filter out attributes that start with "user.syd".
-                // SAFETY: Deny only if the Sandbox is locked for the process.
+            if let Some(mut buf) = buf {
+                // SAFETY: n was returned by the host Linux kernel.
+                unsafe { buf.set_len(n) };
+                buf.shrink_to_fit();
+
+                // SAFETY:
+                // Filter out attributes that start with "user.syd".
+                // Deny only if the Sandbox is locked for the process.
                 let buf = if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
-                    Cow::Owned(filterxattr(&buf[..n], n)?)
+                    Cow::Owned(filterxattr(&buf, n)?)
                 } else {
-                    Cow::Borrowed(&buf[..n])
+                    Cow::Borrowed(&buf)
                 };
 
-                request.write_mem(&buf, req.data.args[1])?;
-                buf.len()
-            } else {
-                n
-            };
+                n = request.write_mem(&buf, req.data.args[1])?;
+            }
 
             #[expect(clippy::cast_possible_wrap)]
             Ok(request.return_syscall(n as i64))
@@ -649,9 +651,8 @@ fn syscall_getxattr_handler(
     // and allocating a Vector of that capacity may overflow.
     let len = to_len_cap(req.data.args[3], XATTR_SIZE_MAX)?;
     let mut buf = if len > 0 {
-        let mut buf = Vec::new();
+        let mut buf: Vec<u8> = Vec::new();
         buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
-        buf.resize(len, 0);
         Some(buf)
     } else {
         None
@@ -675,7 +676,7 @@ fn syscall_getxattr_handler(
         .with_nix_path(|cstr| unsafe { libc::lgetxattr(cstr.as_ptr(), name, ptr.cast(), len) })?;
 
     #[expect(clippy::cast_sign_loss)]
-    let n = match Errno::result(res) {
+    let mut n = match Errno::result(res) {
         Ok(n) => n as usize,
         Err(Errno::ERANGE) if len == XATTR_SIZE_MAX => {
             // SAFETY: Avoid a well-behaving process from
@@ -686,8 +687,12 @@ fn syscall_getxattr_handler(
         Err(errno) => return Err(errno),
     };
 
-    if let Some(buf) = buf {
-        request.write_mem(&buf[..n], req.data.args[2])?;
+    if let Some(mut buf) = buf {
+        // SAFETY: n was returned by the host Linux kernel.
+        unsafe { buf.set_len(n) };
+        buf.shrink_to_fit();
+
+        n = request.write_mem(&buf, req.data.args[2])?;
     }
 
     #[expect(clippy::cast_possible_wrap)]
@@ -728,7 +733,7 @@ fn syscall_getxattrat_handler(
     };
 
     // Read the remote data structure.
-    request.read_mem(buf, req.data.args[4])?;
+    request.read_mem(buf, req.data.args[4], buf.len())?;
 
     // SAFETY: read_mem() has initialized `args` if it succeeded.
     let args = unsafe { args.assume_init() };
@@ -763,7 +768,6 @@ fn syscall_getxattrat_handler(
     let mut buf = if len > 0 {
         let mut buf = Vec::new();
         buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
-        buf.resize(len, 0);
         Some(buf)
     } else {
         None
@@ -787,20 +791,24 @@ fn syscall_getxattrat_handler(
     // SAFETY:
     // 1. `name` is a valid raw pointer (may be NULL)!
     // 2. We do not resolve symbolic links here!
-    let n = match unsafe { getxattrat(fd, base, name, &mut my_args, AtFlags::AT_SYMLINK_NOFOLLOW) }
-    {
-        Ok(n) => n,
-        Err(Errno::ERANGE) if len == XATTR_SIZE_MAX => {
-            // SAFETY: Avoid a well-behaving process from
-            // repeating calls to potentially exhaust memory.
-            // See tar's tests for an example.
-            return Err(Errno::E2BIG);
-        }
-        Err(errno) => return Err(errno),
-    };
+    let mut n =
+        match unsafe { getxattrat(fd, base, name, &mut my_args, AtFlags::AT_SYMLINK_NOFOLLOW) } {
+            Ok(n) => n,
+            Err(Errno::ERANGE) if len == XATTR_SIZE_MAX => {
+                // SAFETY: Avoid a well-behaving process from
+                // repeating calls to potentially exhaust memory.
+                // See tar's tests for an example.
+                return Err(Errno::E2BIG);
+            }
+            Err(errno) => return Err(errno),
+        };
 
-    if let Some(buf) = buf {
-        request.write_mem(&buf[..n], args.value)?;
+    if let Some(mut buf) = buf {
+        // SAFETY: n was returned by the host Linux kernel.
+        unsafe { buf.set_len(n) };
+        buf.shrink_to_fit();
+
+        n = request.write_mem(&buf, args.value)?;
     }
 
     #[expect(clippy::cast_possible_wrap)]
@@ -845,12 +853,9 @@ fn syscall_setxattr_handler(
         (None, 0)
     } else {
         let len = to_len_val(req.data.args[3], XATTR_SIZE_MAX)?;
-        let mut buf = Vec::new();
-        buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
-        buf.resize(len, 0);
-        let n = request.read_mem(&mut buf, req.data.args[2])?;
-        buf.truncate(n);
-        (Some(buf), n)
+        let buf = request.read_vec(req.data.args[2], len)?;
+        let len = buf.len();
+        (Some(buf), len)
     };
     let buf = buf.as_ref().map_or(std::ptr::null(), |b| b.as_ptr()) as *const libc::c_void;
 
@@ -896,7 +901,7 @@ fn syscall_setxattrat_handler(
     };
 
     // Read the remote data structure.
-    request.read_mem(buf, req.data.args[4])?;
+    request.read_mem(buf, req.data.args[4], buf.len())?;
 
     // SAFETY: read_mem() has initialized `args` if it succeeded.
     let args = unsafe { args.assume_init() };
@@ -926,12 +931,9 @@ fn syscall_setxattrat_handler(
         (None, 0)
     } else {
         let len = to_len_val(args.size.into(), XATTR_SIZE_MAX)?;
-        let mut buf = Vec::new();
-        buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
-        buf.resize(len, 0);
-        let n = request.read_mem(&mut buf, args.value)?;
-        buf.truncate(n);
-        (Some(buf), n)
+        let buf = request.read_vec(args.value, len)?;
+        let len = buf.len();
+        (Some(buf), len)
     };
     let buf = buf.as_ref().map_or(std::ptr::null(), |b| b.as_ptr()) as *const libc::c_void;
 
@@ -979,7 +981,6 @@ fn syscall_listxattr_handler(
     let mut buf = if len > 0 {
         let mut buf = Vec::new();
         buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
-        buf.resize(len, 0);
         Some(buf)
     } else {
         None
@@ -1013,17 +1014,21 @@ fn syscall_listxattr_handler(
         Err(errno) => return Err(errno),
     };
 
-    if let Some(buf) = buf {
-        // SAFETY: Filter out attributes that start with "user.syd".
-        // SAFETY: Deny only if the Sandbox is locked for the process.
+    if let Some(mut buf) = buf {
+        // SAFETY: n was returned by the host Linux kernel.
+        unsafe { buf.set_len(n) };
+        buf.shrink_to_fit();
+
+        // SAFETY:
+        // Filter out attributes that start with "user.syd".
+        // Deny only if the Sandbox is locked for the process.
         let buf = if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
-            Cow::Owned(filterxattr(&buf[..n], n)?)
+            Cow::Owned(filterxattr(&buf, n)?)
         } else {
-            Cow::Borrowed(&buf[..n])
+            Cow::Borrowed(&buf)
         };
 
-        request.write_mem(&buf, req.data.args[1])?;
-        n = buf.len();
+        n = request.write_mem(&buf, req.data.args[1])?;
     }
 
     #[expect(clippy::cast_possible_wrap)]
@@ -1055,7 +1060,6 @@ fn syscall_listxattrat_handler(
     let mut buf = if len > 0 {
         let mut buf = Vec::new();
         buf.try_reserve(len).or(Err(Errno::ENOMEM))?;
-        buf.resize(len, 0);
         Some(buf)
     } else {
         None
@@ -1084,17 +1088,21 @@ fn syscall_listxattrat_handler(
         Err(errno) => return Err(errno),
     };
 
-    if let Some(buf) = buf {
-        // SAFETY: Filter out attributes that start with "user.syd".
-        // SAFETY: Deny only if the Sandbox is locked for the process.
+    if let Some(mut buf) = buf {
+        // SAFETY: n was returned by the host Linux kernel.
+        unsafe { buf.set_len(n) };
+        buf.shrink_to_fit();
+
+        // SAFETY:
+        // Filter out attributes that start with "user.syd".
+        // Deny only if the Sandbox is locked for the process.
         let buf = if !sandbox.flags.allow_unsafe_xattr() && sandbox.locked_for(req.pid()) {
-            Cow::Owned(filterxattr(&buf[..n], n)?)
+            Cow::Owned(filterxattr(&buf, n)?)
         } else {
-            Cow::Borrowed(&buf[..n])
+            Cow::Borrowed(&buf)
         };
 
-        request.write_mem(&buf, req.data.args[3])?;
-        n = buf.len();
+        n = request.write_mem(&buf, req.data.args[3])?;
     }
 
     #[expect(clippy::cast_possible_wrap)]
