@@ -142,12 +142,16 @@ pub mod fs;
 pub mod hash;
 /// Secure computing hooks
 pub mod hook;
+/// I/O utilities
+pub mod io;
 /// ioctl(2) request decoder
 pub mod ioctl;
 /// Landlock policy helper library for Syd
 pub mod landlock_policy;
 /// Simple logging on standard error using JSON lines
 pub mod log;
+/// Path lookup and canonicalization utilities
+pub mod lookup;
 /// magic symlink utilities
 pub mod magic;
 /// Interface to Open File Description locks
@@ -190,6 +194,8 @@ pub mod uts;
 pub mod wildmatch;
 /// Interface to wordexp(3)
 pub mod wordexp;
+/// Extended attribute utilities
+pub mod xattr;
 
 // Vendored crates:
 /// Interface to Linux capabilities
