@@ -24,6 +24,7 @@ use std::{
 };
 
 use btoi::btoi;
+use itoa::Integer;
 use memchr::{
     arch::all::{is_equal, is_prefix, is_suffix},
     memchr, memmem, memrchr,
@@ -875,16 +876,8 @@ impl XPath {
 
     /// Check if path points to machine-id(5), /etc/hostid or /var/adm/hostid.
     pub fn is_machine_id(&self) -> bool {
-        for f in [
-            &b"/etc/machine-id"[..],
-            &b"/etc/hostid"[..],
-            &b"/var/adm/hostid"[..],
-        ] {
-            if self.is_equal(f) {
-                return true;
-            }
-        }
-        false
+        const MACHINE_ID: &[&[u8]] = &[b"/etc/machine-id", b"/etc/hostid", b"/var/adm/hostid"];
+        MACHINE_ID.iter().any(|f| self.is_equal(f))
     }
 
     /// Check if path points to a proc_pid_status(5) file.
@@ -1205,28 +1198,116 @@ impl XPathBuf {
         self.0.drain(range);
     }
 
-    /// Create a path from the given PID.
-    pub fn from_pid(pid: Pid) -> Self {
+    /// Create a root magiclink path from the given PID.
+    pub fn from_root(pid: Pid) -> Result<Self, Errno> {
         let mut buf = itoa::Buffer::new();
-        buf.format(pid.as_raw()).as_bytes().into()
+        let mut pfd = Vec::new();
+        let len = pid_t::MAX_STR_LEN
+            .checked_add("/root".len())
+            .ok_or(Errno::EOVERFLOW)?;
+        pfd.try_reserve(len).or(Err(Errno::ENOMEM))?;
+        pfd.extend_from_slice(buf.format(pid.as_raw()).as_bytes());
+        pfd.extend_from_slice(b"/root");
+        pfd.shrink_to_fit();
+        Ok(pfd.into())
+    }
+
+    /// Create a exe magiclink path from the given PID.
+    pub fn from_exe(pid: Pid) -> Result<Self, Errno> {
+        let mut buf = itoa::Buffer::new();
+        let mut pfd = Vec::new();
+        let len = pid_t::MAX_STR_LEN
+            .checked_add("/exe".len())
+            .ok_or(Errno::EOVERFLOW)?;
+        pfd.try_reserve(len).or(Err(Errno::ENOMEM))?;
+        pfd.extend_from_slice(buf.format(pid.as_raw()).as_bytes());
+        pfd.extend_from_slice(b"/exe");
+        pfd.shrink_to_fit();
+        Ok(pfd.into())
+    }
+
+    /// Create a cwd magiclink path from the given PID.
+    pub fn from_cwd(pid: Pid) -> Result<Self, Errno> {
+        let mut buf = itoa::Buffer::new();
+        let mut pfd = Vec::new();
+        let len = pid_t::MAX_STR_LEN
+            .checked_add("/cwd".len())
+            .ok_or(Errno::EOVERFLOW)?;
+        pfd.try_reserve(len).or(Err(Errno::ENOMEM))?;
+        pfd.extend_from_slice(buf.format(pid.as_raw()).as_bytes());
+        pfd.extend_from_slice(b"/cwd");
+        pfd.shrink_to_fit();
+        Ok(pfd.into())
+    }
+
+    /// Create a path from the given PID.
+    pub fn from_pid(pid: Pid) -> Result<Self, Errno> {
+        let mut buf = itoa::Buffer::new();
+        let mut pfd = Vec::new();
+        pfd.try_reserve(pid_t::MAX_STR_LEN).or(Err(Errno::ENOMEM))?;
+        pfd.extend_from_slice(buf.format(pid.as_raw()).as_bytes());
+        pfd.shrink_to_fit();
+        Ok(pfd.into())
+    }
+
+    /// Create a path from the given TGID and TID.
+    pub fn from_task(tgid: Pid, tid: Pid) -> Result<Self, Errno> {
+        let mut buf = itoa::Buffer::new();
+        let mut pfd = Vec::new();
+        let len = pid_t::MAX_STR_LEN
+            .checked_mul(2)
+            .ok_or(Errno::EOVERFLOW)?
+            .checked_add("/task/".len())
+            .ok_or(Errno::EOVERFLOW)?;
+        pfd.try_reserve(len).or(Err(Errno::ENOMEM))?;
+        pfd.extend_from_slice(buf.format(tgid.as_raw()).as_bytes());
+        pfd.extend_from_slice(b"/task/");
+        pfd.extend_from_slice(buf.format(tid.as_raw()).as_bytes());
+        pfd.shrink_to_fit();
+        Ok(pfd.into())
     }
 
     /// Create a path from the given FD.
-    pub fn from_fd(fd: RawFd) -> Self {
+    pub fn from_fd(fd: RawFd) -> Result<Self, Errno> {
         let mut buf = itoa::Buffer::new();
-        buf.format(fd).as_bytes().into()
+        let mut pfd = Vec::new();
+        pfd.try_reserve(RawFd::MAX_STR_LEN).or(Err(Errno::ENOMEM))?;
+        pfd.extend_from_slice(buf.format(fd).as_bytes());
+        Ok(pfd.into())
+    }
+
+    /// Create a path from the given PID and FD.
+    pub fn from_pid_fd(pid: Pid, fd: RawFd) -> Result<Self, Errno> {
+        let mut buf = itoa::Buffer::new();
+        let mut pfd = Vec::new();
+        let len = pid_t::MAX_STR_LEN
+            .checked_add(RawFd::MAX_STR_LEN)
+            .ok_or(Errno::EOVERFLOW)?
+            .checked_add("/fd/".len())
+            .ok_or(Errno::EOVERFLOW)?;
+        pfd.try_reserve(len).or(Err(Errno::ENOMEM))?;
+        pfd.extend_from_slice(buf.format(pid.as_raw()).as_bytes());
+        pfd.extend_from_slice(b"/fd/");
+        pfd.extend_from_slice(buf.format(fd).as_bytes());
+        pfd.shrink_to_fit();
+        Ok(pfd.into())
     }
 
     /// Create a path for the given self-FD.
     ///
     /// Used for _procfs_(5) indirection.
-    pub fn from_self_fd(fd: RawFd) -> Self {
+    pub fn from_self_fd(fd: RawFd) -> Result<Self, Errno> {
         // SAFETY:
         // Use /proc/thread-self rather than /proc/self
         // because CLONE_FILES may be in effect!
-        let mut pfd = Self::from("thread-self/fd");
+        const LEN: usize = "thread-self/fd".len() + 1 + RawFd::MAX_STR_LEN;
+        let mut pfd = Vec::new();
+        pfd.try_reserve(LEN).or(Err(Errno::ENOMEM))?;
+        pfd.extend_from_slice(b"thread-self/fd");
+        let mut pfd: Self = pfd.into();
         pfd.push_fd(fd);
-        pfd
+        pfd.shrink_to_fit();
+        Ok(pfd)
     }
 
     /// Append the formatted FD as a new component.
@@ -1321,6 +1402,14 @@ impl XPathBuf {
         self.0.shrink_to_fit()
     }
 
+    /// Try to clone an `XPathBuf`, returning `Err(Errno::ENOMEM)` on allocation errors.
+    pub fn try_clone(&self) -> Result<Self, Errno> {
+        let mut vec = Vec::new();
+        vec.try_reserve(self.len()).or(Err(Errno::ENOMEM))?;
+        vec.extend_from_slice(self.as_bytes());
+        Ok(vec.into())
+    }
+
     /// Reserve capacity for additional more bytes to be inserted.
     /// May reserve more space to avoid frequent allocations.
     pub fn try_reserve(&mut self, additional: usize) -> Result<(), Errno> {
