@@ -476,7 +476,8 @@ fn usize_to_ne_bytes(value: usize, sizeof_ptr: usize) -> Vec<u8> {
 
 /// Return the stat of the given process.
 pub fn proc_stat(pid: Pid) -> Result<Stat, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/stat".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"stat");
 
     // Safely open proc(5) file.
@@ -488,7 +489,8 @@ pub fn proc_stat(pid: Pid) -> Result<Stat, Errno> {
 
 /// Return the memory stat of the given process.
 pub fn proc_statm(pid: Pid) -> Result<Statm, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/statm".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"statm");
 
     // Safely open proc(5) file.
@@ -506,7 +508,8 @@ pub fn proc_status(pid: Pid) -> Result<Status, Errno> {
 
 /// Open a FD to /proc/$pid/status and return an OwnedFd.
 pub fn proc_status_open(pid: Pid) -> Result<OwnedFd, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/status".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"status");
 
     safe_open_proc(&path).map_err(proc_errno)
@@ -573,7 +576,8 @@ pub fn proc_stack_start(pid: Pid) -> Result<u64, Errno> {
 ///
 /// If the process is "running", it returns `Errno::EBUSY`.
 pub fn proc_stack_pointer(pid: Pid) -> Result<u64, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/syscall".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"syscall");
 
     // Safely open proc(5) file.
@@ -616,7 +620,8 @@ pub fn proc_stack_pointer(pid: Pid) -> Result<u64, Errno> {
 
 /// Get the auxiliary vector of the given `Pid`.
 pub fn proc_auxv(pid: Pid) -> Result<SydHashMap<u64, u64>, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/auxv".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"auxv");
 
     // Safely open proc(5) file.
@@ -652,15 +657,15 @@ pub fn proc_auxv(pid: Pid) -> Result<SydHashMap<u64, u64>, Errno> {
 ///
 /// This function reads the symbolic link `/proc/<pid>/cwd` to determine the CWD.
 pub fn proc_cwd(pid: Pid) -> Result<XPathBuf, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
-    path.push(b"cwd");
+    let path = XPathBuf::from_cwd(pid)?;
     readlinkat(PROC_FILE(), &path).map_err(proc_errno)
 }
 
 /// Retrieves the command name (comm) of the specified process ID (PID)
 /// as a single string.
 pub fn proc_comm(pid: Pid) -> Result<XPathBuf, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/comm".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"comm");
 
     // Safely open proc(5) file.
@@ -688,7 +693,8 @@ pub fn proc_comm(pid: Pid) -> Result<XPathBuf, Errno> {
 /// bytes (`'\0'`) with spaces to format the command line as a readable string.
 pub fn proc_cmdline(pid: Pid) -> Result<XPathBuf, Errno> {
     // Construct path to the appropriate cmdline file.
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/cmdline".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"cmdline");
 
     // Safely open proc(5) file.
@@ -738,7 +744,8 @@ pub fn proc_maps(pid: Pid) -> Result<Vec<SydMemoryMap>, Errno> {
 
 /// Open a FD to _proc_pid_maps_(5) and return an OwnedFd.
 pub fn proc_maps_open(pid: Pid) -> Result<OwnedFd, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/maps".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"maps");
 
     safe_open_proc(&path).map_err(proc_errno)
@@ -756,7 +763,8 @@ pub fn proc_maps_read(fd: OwnedFd) -> Result<Vec<SydMemoryMap>, Errno> {
 
 /// Return the memory maps of the given process.
 pub fn proc_smaps(pid: Pid) -> Result<Vec<SydMemoryMap>, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/smaps".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"smaps");
 
     // Safely open proc(5) file.
@@ -776,7 +784,9 @@ pub fn proc_smaps(pid: Pid) -> Result<Vec<SydMemoryMap>, Errno> {
 /// This is more efficient than using `proc_smaps` and iterating over maps.
 /// See: https://www.kernel.org/doc/Documentation/ABI/testing/procfs-smaps_rollup
 pub fn proc_smaps_rollup(pid: Pid) -> Result<SydMemoryMap, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/smaps_rollup".len())
+        .or(Err(Errno::ENOMEM))?;
     path.push(b"smaps_rollup");
 
     // Safely open proc(5) file.
@@ -1330,7 +1340,8 @@ pub fn proc_environ(pid: Pid) -> Result<SydHashMap<OsString, OsString>, Errno> {
 
 /// Open a FD to _proc_pid_environ_(5) and return an OwnedFd.
 pub fn proc_environ_open(pid: Pid) -> Result<OwnedFd, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/environ".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"environ");
 
     safe_open_proc(&path).map_err(proc_errno)
@@ -1450,7 +1461,9 @@ pub fn proc_crypto_read(fd: OwnedFd) -> Result<CryptoTable, Errno> {
 ///
 /// Returns the inodes of all UNIX domain sockets.
 pub fn proc_unix_inodes(pid: Pid) -> Result<SydHashSet<u64>, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/net/unix".len())
+        .or(Err(Errno::ENOMEM))?;
     path.push(b"net");
     path.push(b"unix");
 
@@ -1492,7 +1505,8 @@ pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType) -> Result<(), Errno> {
     // This function requires CONFIG_CROSS_MEMORY_ATTACH,
     // unless Syd is explicitly configured to fallback.
     let mut proc_mem = if Sandbox::use_proc_pid_mem() {
-        let mut path = XPathBuf::from_pid(pid);
+        let mut path = XPathBuf::from_pid(pid)?;
+        path.try_reserve(b"/mem".len()).or(Err(Errno::ENOMEM))?;
         path.push(b"mem");
 
         let mut flags = OFLAG_PROC;
@@ -1767,7 +1781,8 @@ pub fn proc_set_at_secure(pid: Pid, elf_type: ElfType) -> Result<(), Errno> {
 pub fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
     let pid = pid.unwrap_or_else(getpid);
 
-    let mut dir = XPathBuf::from_pid(pid);
+    let mut dir = XPathBuf::from_pid(pid)?;
+    dir.try_reserve(b"/fd".len()).or(Err(Errno::ENOMEM))?;
     dir.push(b"fd");
 
     let dir = safe_open(
@@ -1809,7 +1824,8 @@ pub fn proc_pid_fd(pid: Option<Pid>) -> Result<Vec<(RawFd, XPathBuf)>, Errno> {
 
 /// Return the max open files soft limit for the given process.
 pub fn proc_max_open_files(pid: Pid) -> Result<LimitValue, Errno> {
-    let mut path = XPathBuf::from_pid(pid);
+    let mut path = XPathBuf::from_pid(pid)?;
+    path.try_reserve(b"/limits".len()).or(Err(Errno::ENOMEM))?;
     path.push(b"limits");
 
     // Safely open proc(5) file.
