@@ -39,6 +39,7 @@ use nix::{
     NixPath,
 };
 use serde::{ser::SerializeMap, Serialize};
+use zeroize::Zeroizing;
 
 use crate::{
     compat::{fstatx, timespec_tv_nsec_t, TimeSpec32, TimeSpec64, STATX_INO, XATTR_NAME_MAX},
@@ -413,6 +414,108 @@ impl RemoteProcess {
         Ok((canonical_path, magic, doterr, empty_path))
     }
 
+    /// Allocate and read a `Zeroizing` buffer from remote process's memory with `process_vm_readv()`.
+    ///
+    /// # Safety
+    ///
+    /// This function is unsafe because the request is not validated.
+    pub(crate) unsafe fn read_vec_zeroed(
+        &self,
+        remote_addr: u64,
+        len: usize,
+    ) -> Result<Zeroizing<Vec<u8>>, Errno> {
+        if Sandbox::use_proc_pid_mem() {
+            return self.read_vec_zeroed_proc(remote_addr, len);
+        }
+
+        let mut local_buffer = Zeroizing::new(Vec::new());
+
+        // Check for zero length and return an empty Vector.
+        if len == 0 {
+            return Ok(local_buffer);
+        }
+
+        // SAFETY: Check pointer against mmap_min_addr before allocation,
+        // but after length is zero check.
+        if remote_addr < *MMAP_MIN_ADDR {
+            return Err(Errno::EFAULT);
+        }
+        let remote_addr = usize::try_from(remote_addr).or(Err(Errno::EFAULT))?;
+
+        local_buffer.try_reserve(len).or(Err(Errno::ENOMEM))?;
+
+        // SAFETY: We are using `set_len(len)` after reserving enough
+        // capacity with `try_reserve(len)`. This is safe because the
+        // buffer was already allocated with enough memory to hold `len`
+        // elements, and we are not exceeding the reserved capacity. The
+        // memory is valid for `len` elements.
+        unsafe { local_buffer.set_len(len) };
+
+        let len = process_vm_readv(
+            self.pid,
+            &mut [IoSliceMut::new(&mut local_buffer)],
+            &[RemoteIoVec {
+                len,
+                base: remote_addr,
+            }],
+        )?;
+
+        // SAFETY: len is returned by the Linux kernel.
+        unsafe { local_buffer.set_len(len) };
+        local_buffer.shrink_to_fit();
+
+        Ok(local_buffer)
+    }
+
+    /// Allocate and read a buffer from remote process's memory with `process_vm_readv()`.
+    ///
+    /// # Safety
+    ///
+    /// This function is unsafe because the request is not validated.
+    pub(crate) unsafe fn read_vec(&self, remote_addr: u64, len: usize) -> Result<Vec<u8>, Errno> {
+        if Sandbox::use_proc_pid_mem() {
+            return self.read_vec_proc(remote_addr, len);
+        }
+
+        let mut local_buffer = Vec::new();
+
+        // Check for zero length and return an empty Vector.
+        if len == 0 {
+            return Ok(local_buffer);
+        }
+
+        // SAFETY: Check pointer against mmap_min_addr before allocation,
+        // but after length is zero check.
+        if remote_addr < *MMAP_MIN_ADDR {
+            return Err(Errno::EFAULT);
+        }
+        let remote_addr = usize::try_from(remote_addr).or(Err(Errno::EFAULT))?;
+
+        local_buffer.try_reserve(len).or(Err(Errno::ENOMEM))?;
+
+        // SAFETY: We are using `set_len(len)` after reserving enough
+        // capacity with `try_reserve(len)`. This is safe because the
+        // buffer was already allocated with enough memory to hold `len`
+        // elements, and we are not exceeding the reserved capacity. The
+        // memory is valid for `len` elements.
+        unsafe { local_buffer.set_len(len) };
+
+        let len = process_vm_readv(
+            self.pid,
+            &mut [IoSliceMut::new(&mut local_buffer)],
+            &[RemoteIoVec {
+                len,
+                base: remote_addr,
+            }],
+        )?;
+
+        // SAFETY: len is returned by the Linux kernel.
+        unsafe { local_buffer.set_len(len) };
+        local_buffer.shrink_to_fit();
+
+        Ok(local_buffer)
+    }
+
     /// Read data from remote process's memory with `process_vm_readv()`.
     ///
     /// # Safety
@@ -422,27 +525,148 @@ impl RemoteProcess {
         &self,
         local_buffer: &mut [u8],
         remote_addr: u64,
+        len: usize,
     ) -> Result<usize, Errno> {
         if Sandbox::use_proc_pid_mem() {
-            return self.read_mem_proc(local_buffer, remote_addr);
+            return self.read_mem_proc(local_buffer, remote_addr, len);
         }
 
         // SAFETY: Check pointer against mmap_min_addr.
         if remote_addr < *MMAP_MIN_ADDR {
             return Err(Errno::EFAULT);
         }
+        let remote_addr = usize::try_from(remote_addr).or(Err(Errno::EFAULT))?;
 
-        let len = local_buffer.len();
         process_vm_readv(
             self.pid,
             &mut [IoSliceMut::new(local_buffer)],
             &[RemoteIoVec {
                 len,
-                base: usize::try_from(remote_addr).or(Err(Errno::EFAULT))?,
+                base: remote_addr,
             }],
         )
     }
 
+    /// Fallback method to allocate and read a `Zeroizing` buffer from `/proc/$pid/mem` when `process_vm_readv()` is unavailable.
+    ///
+    /// # Safety
+    ///
+    /// This function is unsafe because the request is not validated.
+    pub(crate) unsafe fn read_vec_zeroed_proc(
+        &self,
+        remote_addr: u64,
+        len: usize,
+    ) -> Result<Zeroizing<Vec<u8>>, Errno> {
+        let mut local_buffer = Zeroizing::new(Vec::new());
+
+        // Check for zero length and return an empty Vector.
+        if len == 0 {
+            return Ok(local_buffer);
+        }
+
+        // SAFETY: Check pointer against mmap_min_addr before allocation,
+        // but after length is zero check.
+        if remote_addr < *MMAP_MIN_ADDR {
+            return Err(Errno::EFAULT);
+        }
+
+        local_buffer.try_reserve(len).or(Err(Errno::ENOMEM))?;
+
+        // SAFETY: We are using `set_len(len)` after reserving enough
+        // capacity with `try_reserve(len)`. This is safe because the
+        // buffer was already allocated with enough memory to hold `len`
+        // elements, and we are not exceeding the reserved capacity. The
+        // memory is valid for `len` elements.
+        unsafe { local_buffer.set_len(len) };
+
+        let mut path = XPathBuf::from_pid(self.pid)?;
+        path.try_reserve(b"/mem".len()).or(Err(Errno::ENOMEM))?;
+        path.push(b"mem");
+
+        let mut file = safe_open_msym(PROC_FILE(), &path, OFlag::O_RDONLY, ResolveFlag::empty())
+            .map(File::from)
+            .or(Err(Errno::EACCES))?;
+        file.seek(SeekFrom::Start(remote_addr))
+            .or(Err(Errno::EACCES))?;
+
+        let mut nread = 0;
+        #[expect(clippy::arithmetic_side_effects)]
+        while nread < len {
+            match file.read(&mut local_buffer[nread..]) {
+                Ok(0) => return Err(Errno::EACCES),
+                Ok(n) => nread += n,
+                Err(ref e) if e.kind() == io::ErrorKind::Interrupted => {}
+                Err(_) => return Err(Errno::EACCES),
+            }
+        }
+
+        // SAFETY: nread is returned by the Linux kernel.
+        unsafe { local_buffer.set_len(nread) };
+        local_buffer.shrink_to_fit();
+
+        Ok(local_buffer)
+    }
+
+    /// Fallback method to allocate and read a buffer from `/proc/$pid/mem` when `process_vm_readv()` is unavailable.
+    ///
+    /// # Safety
+    ///
+    /// This function is unsafe because the request is not validated.
+    pub(crate) unsafe fn read_vec_proc(
+        &self,
+        remote_addr: u64,
+        len: usize,
+    ) -> Result<Vec<u8>, Errno> {
+        let mut local_buffer = Vec::new();
+
+        // Check for zero length and return an empty Vector.
+        if len == 0 {
+            return Ok(local_buffer);
+        }
+
+        // SAFETY: Check pointer against mmap_min_addr before allocation,
+        // but after length is zero check.
+        if remote_addr < *MMAP_MIN_ADDR {
+            return Err(Errno::EFAULT);
+        }
+
+        local_buffer.try_reserve(len).or(Err(Errno::ENOMEM))?;
+
+        // SAFETY: We are using `set_len(len)` after reserving enough
+        // capacity with `try_reserve(len)`. This is safe because the
+        // buffer was already allocated with enough memory to hold `len`
+        // elements, and we are not exceeding the reserved capacity. The
+        // memory is valid for `len` elements.
+        unsafe { local_buffer.set_len(len) };
+
+        let mut path = XPathBuf::from_pid(self.pid)?;
+        path.try_reserve(b"/mem".len()).or(Err(Errno::ENOMEM))?;
+        path.push(b"mem");
+
+        let mut file = safe_open_msym(PROC_FILE(), &path, OFlag::O_RDONLY, ResolveFlag::empty())
+            .map(File::from)
+            .or(Err(Errno::EACCES))?;
+        file.seek(SeekFrom::Start(remote_addr))
+            .or(Err(Errno::EACCES))?;
+
+        let mut nread = 0;
+        #[expect(clippy::arithmetic_side_effects)]
+        while nread < len {
+            match file.read(&mut local_buffer[nread..]) {
+                Ok(0) => return Err(Errno::EACCES),
+                Ok(n) => nread += n,
+                Err(ref e) if e.kind() == io::ErrorKind::Interrupted => {}
+                Err(_) => return Err(Errno::EACCES),
+            }
+        }
+
+        // SAFETY: nread is returned by the Linux kernel.
+        unsafe { local_buffer.set_len(nread) };
+        local_buffer.shrink_to_fit();
+
+        Ok(local_buffer)
+    }
+
     /// Fallback method to read data from `/proc/$pid/mem` when `process_vm_readv()` is unavailable.
     ///
     /// # Safety
@@ -452,13 +676,15 @@ impl RemoteProcess {
         &self,
         local_buffer: &mut [u8],
         remote_addr: u64,
+        len: usize,
     ) -> Result<usize, Errno> {
         // SAFETY: Check pointer against mmap_min_addr.
         if remote_addr < *MMAP_MIN_ADDR {
             return Err(Errno::EFAULT);
         }
 
-        let mut path = XPathBuf::from_pid(self.pid);
+        let mut path = XPathBuf::from_pid(self.pid)?;
+        path.try_reserve(b"/mem".len()).or(Err(Errno::ENOMEM))?;
         path.push(b"mem");
 
         let mut file = safe_open_msym(PROC_FILE(), &path, OFlag::O_RDONLY, ResolveFlag::empty())
@@ -469,7 +695,7 @@ impl RemoteProcess {
 
         let mut nread = 0;
         #[expect(clippy::arithmetic_side_effects)]
-        while nread < local_buffer.len() {
+        while nread < len {
             match file.read(&mut local_buffer[nread..]) {
                 Ok(0) => return Err(Errno::EACCES),
                 Ok(n) => nread += n,
@@ -531,7 +757,8 @@ impl RemoteProcess {
             return Ok(0);
         }
 
-        let mut path = XPathBuf::from_pid(self.pid);
+        let mut path = XPathBuf::from_pid(self.pid)?;
+        path.try_reserve(b"/mem".len()).or(Err(Errno::ENOMEM))?;
         path.push(b"mem");
 
         let mut file = safe_open_msym(PROC_FILE(), &path, OFlag::O_WRONLY, ResolveFlag::empty())
@@ -582,7 +809,7 @@ impl RemoteProcess {
             // SAFETY:
             // 1. Assume error on zero-read.
             // 2. Validate the request after memory read.
-            let len = unsafe { self.read_mem(ptr, addr + off as u64) }?;
+            let len = unsafe { self.read_mem(ptr, addr + off as u64, len) }?;
             if len == 0 {
                 return Err(Errno::EFAULT);
             }
@@ -597,7 +824,13 @@ impl RemoteProcess {
 
                 // Adjust to actual size up to NUL-byte.
                 off += nul;
-                return Ok(buf[..off].into());
+
+                // Allocate vector on heap gracefully.
+                let mut vec = Vec::new();
+                vec.try_reserve(off).or(Err(Errno::ENOMEM))?;
+                vec.extend_from_slice(&buf[..off]);
+
+                return Ok(vec.into());
             }
 
             off += len;
@@ -674,7 +907,7 @@ impl Serialize for UNotifyEventRequest {
             // specified address.
             if let Some(addr) = auxv.get(&(libc::AT_RANDOM as u64)) {
                 let mut at_random = [0u8; 16];
-                if *addr >= *MMAP_MIN_ADDR && self.read_mem(&mut at_random, *addr).is_ok() {
+                if *addr >= *MMAP_MIN_ADDR && self.read_mem(&mut at_random, *addr, 16).is_ok() {
                     map.serialize_entry("at_random", &HEXLOWER.encode(&at_random))?;
                 }
             }
@@ -710,12 +943,12 @@ impl Serialize for UNotifyEventRequest {
         let mut ip_read = false;
         let mut sp_read = false;
 
-        if ip >= *MMAP_MIN_ADDR && self.read_mem(&mut ip_mem, ip).is_ok() {
+        if ip >= *MMAP_MIN_ADDR && self.read_mem(&mut ip_mem, ip, 64).is_ok() {
             ip_read = true;
         }
 
         if let Some(sp) = sp {
-            if sp >= *MMAP_MIN_ADDR && self.read_mem(&mut sp_mem, sp).is_ok() {
+            if sp >= *MMAP_MIN_ADDR && self.read_mem(&mut sp_mem, sp, 64).is_ok() {
                 sp_read = true;
             }
         }
@@ -920,16 +1153,13 @@ impl UNotifyEventRequest {
     ///
     /// Name must be a NUL-terminated string or `Err(Errno::ERANGE)` is returned.
     pub(crate) fn read_xattr(&self, addr: u64) -> Result<CString, Errno> {
-        let mut buf = Vec::new();
-
-        buf.try_reserve(XATTR_NAME_MAX).or(Err(Errno::ENOMEM))?;
-        buf.resize(XATTR_NAME_MAX, 0);
-
-        let mut n = self.read_mem(&mut buf, addr)?;
-
-        let z = memchr(0, &buf[..n]).ok_or(Errno::ERANGE)?;
-        n = n.min(z.checked_add(1).ok_or(Errno::ERANGE)?);
-        buf.truncate(n);
+        let mut buf = self.read_vec(addr, XATTR_NAME_MAX)?;
+        let z = memchr(0, &buf)
+            .ok_or(Errno::ERANGE)?
+            .checked_add(1)
+            .ok_or(Errno::ERANGE)?;
+        buf.truncate(z);
+        buf.shrink_to_fit();
 
         // Check for empty name.
         let len = buf.len(); // Includes NUL-byte.
@@ -989,7 +1219,7 @@ impl UNotifyEventRequest {
             // Read remote memory.
             //
             // SAFETY: The request is going to be validated.
-            let n = unsafe { process.read_mem(slice, read_addr) }?;
+            let n = unsafe { process.read_mem(slice, read_addr, slice.len()) }?;
 
             // SAFETY: Assume error on zero-read.
             if n == 0 {
@@ -1041,10 +1271,7 @@ impl UNotifyEventRequest {
 
         // Allocate buffer.
         // Size is already capped to page size.
-        let mut raw = Vec::new();
-        raw.try_reserve(size).or(Err(Errno::ENOMEM))?;
-        raw.resize(size, 0);
-        self.read_mem(&mut raw, addr)?;
+        let raw = self.read_vec(addr, size)?;
 
         // SAFETY: Verify trailing bytes are zero; otherwise E2BIG.
         if raw.iter().skip(OPEN_HOW_SIZE_LATEST).any(|&b| b != 0) {
@@ -1078,8 +1305,9 @@ impl UNotifyEventRequest {
             return Err(Errno::EFAULT);
         }
 
-        let mut buf = [0u8; size_of::<libc::utimbuf>()];
-        self.read_mem(&mut buf, addr)?;
+        const LEN: usize = size_of::<libc::utimbuf>();
+        let mut buf = [0u8; LEN];
+        self.read_mem(&mut buf, addr, LEN)?;
 
         // SAFETY: The following unsafe block assumes that:
         // 1. The memory layout of utimbuf in our Rust environment
@@ -1109,8 +1337,9 @@ impl UNotifyEventRequest {
             return Err(Errno::EFAULT);
         }
 
-        let mut buf = [0u8; size_of::<libc::timeval>() * 2];
-        self.read_mem(&mut buf, addr)?;
+        const LEN: usize = size_of::<libc::timeval>() * 2;
+        let mut buf = [0u8; LEN];
+        self.read_mem(&mut buf, addr, LEN)?;
 
         // SAFETY: The following unsafe block assumes that:
         // 1. The memory layout of timeval in our Rust environment
@@ -1152,8 +1381,9 @@ impl UNotifyEventRequest {
             return Err(Errno::EFAULT);
         }
 
-        let mut buf = [0u8; size_of::<TimeSpec32>()];
-        self.read_mem(&mut buf, addr)?;
+        const LEN: usize = size_of::<TimeSpec32>();
+        let mut buf = [0u8; LEN];
+        self.read_mem(&mut buf, addr, LEN)?;
 
         // SAFETY: The following unsafe block assumes that:
         // 1. The memory layout of timespec in our Rust environment
@@ -1188,8 +1418,9 @@ impl UNotifyEventRequest {
             return Err(Errno::EFAULT);
         }
 
-        let mut buf = [0u8; size_of::<TimeSpec64>()];
-        self.read_mem(&mut buf, addr)?;
+        const LEN: usize = size_of::<TimeSpec64>();
+        let mut buf = [0u8; LEN];
+        self.read_mem(&mut buf, addr, LEN)?;
 
         // SAFETY: The following unsafe block assumes that:
         // 1. The memory layout of timespec in our Rust environment
@@ -1222,8 +1453,9 @@ impl UNotifyEventRequest {
             return Err(Errno::EFAULT);
         }
 
-        let mut buf = [0u8; size_of::<TimeSpec32>() * 2];
-        self.read_mem(&mut buf, addr)?;
+        const LEN: usize = size_of::<TimeSpec32>() * 2;
+        let mut buf = [0u8; LEN];
+        self.read_mem(&mut buf, addr, LEN)?;
 
         // SAFETY: The following unsafe block assumes that:
         // 1. The memory layout of timespec in our Rust environment
@@ -1264,8 +1496,9 @@ impl UNotifyEventRequest {
             return Err(Errno::EFAULT);
         }
 
-        let mut buf = [0u8; size_of::<TimeSpec64>() * 2];
-        self.read_mem(&mut buf, addr)?;
+        const LEN: usize = size_of::<TimeSpec64>() * 2;
+        let mut buf = [0u8; LEN];
+        self.read_mem(&mut buf, addr, LEN)?;
 
         // SAFETY: The following unsafe block assumes that:
         // 1. The memory layout of timespec in our Rust environment
@@ -1343,16 +1576,55 @@ impl UNotifyEventRequest {
         }
     }
 
+    /// Read a `Zeroizing` vector from remote process's memory with `process_vm_readv()`.
+    pub(crate) fn read_vec_zeroed(
+        &self,
+        remote_addr: u64,
+        len: usize,
+    ) -> Result<Zeroizing<Vec<u8>>, Errno> {
+        let process = RemoteProcess::new(self.scmpreq.pid());
+
+        // SAFETY: The request is validated.
+        match unsafe { process.read_vec_zeroed(remote_addr, len) } {
+            Ok(vec) => {
+                if self.is_valid() {
+                    Ok(vec)
+                } else {
+                    Err(Errno::ESRCH)
+                }
+            }
+            Err(errno) => Err(errno),
+        }
+    }
+
+    /// Read a vector from remote process's memory with `process_vm_readv()`.
+    pub(crate) fn read_vec(&self, remote_addr: u64, len: usize) -> Result<Vec<u8>, Errno> {
+        let process = RemoteProcess::new(self.scmpreq.pid());
+
+        // SAFETY: The request is validated.
+        match unsafe { process.read_vec(remote_addr, len) } {
+            Ok(vec) => {
+                if self.is_valid() {
+                    Ok(vec)
+                } else {
+                    Err(Errno::ESRCH)
+                }
+            }
+            Err(errno) => Err(errno),
+        }
+    }
+
     /// Read data from remote process's memory with `process_vm_readv()`.
     pub(crate) fn read_mem(
         &self,
         local_buffer: &mut [u8],
         remote_addr: u64,
+        len: usize,
     ) -> Result<usize, Errno> {
         let process = RemoteProcess::new(self.scmpreq.pid());
 
         // SAFETY: The request is validated.
-        match unsafe { process.read_mem(local_buffer, remote_addr) } {
+        match unsafe { process.read_mem(local_buffer, remote_addr, len) } {
             Ok(n) => {
                 if self.is_valid() {
                     Ok(n)
