@@ -24,8 +24,9 @@ use nix::{
 
 use crate::{
     cookie::safe_unlinkat,
-    fs::{is_active_fd, safe_open},
+    fs::is_active_fd,
     log::{now, Tm},
+    lookup::safe_open,
     path::{XPathBuf, PATH_MAX},
     retry::retry_on_eintr,
 };
