@@ -951,29 +951,36 @@ syd::main! {
     }
 
     if sandbox.flags.unshare_time() {
-        let (moffset, boffset) = if let Some(offset) = sandbox.time {
-            (offset, offset)
+        let boffset = if let Some(boffset) = sandbox.boottime {
+            boffset
         } else {
-            let mtime = clock_gettime(ClockId::CLOCK_MONOTONIC)?;
             let btime = clock_gettime(ClockId::CLOCK_BOOTTIME)?;
-            let moffset = mtime.tv_sec().checked_neg().ok_or(Errno::EOVERFLOW)?;
-            let boffset = btime.tv_sec().checked_neg().ok_or(Errno::EOVERFLOW)?;
             // Into is necessary on 32-bit.
             #[expect(clippy::useless_conversion)]
-            (moffset.into(), boffset.into())
+            btime.tv_sec().checked_neg().ok_or(Errno::EOVERFLOW)?.into()
+        };
+        let moffset = if let Some(moffset) = sandbox.monotime {
+            moffset
+        } else {
+            let mtime = clock_gettime(ClockId::CLOCK_MONOTONIC)?;
+            // Into is necessary on 32-bit.
+            #[expect(clippy::useless_conversion)]
+            mtime.tv_sec().checked_neg().ok_or(Errno::EOVERFLOW)?.into()
         };
 
-        // <clock-id> <offset-secs> <offset-nanosecs>
-        let mut buf = itoa::Buffer::new();
-        let mut data = "monotonic ".to_string();
-        data.push_str(buf.format(moffset));
-        data.push_str(" 0\nboottime ");
-        data.push_str(buf.format(boffset));
-        data.push_str(" 0\n");
+        if boffset != 0 || moffset != 0 {
+            // <clock-id> <offset-secs> <offset-nanosecs>
+            let mut buf = itoa::Buffer::new();
+            let mut data = "monotonic ".to_string();
+            data.push_str(buf.format(moffset));
+            data.push_str(" 0\nboottime ");
+            data.push_str(buf.format(boffset));
+            data.push_str(" 0\n");
 
-        proc_write(pid, b"timens_offsets", data.as_bytes())?;
-        info!("ctx": "set_boot_time", "msg": "set boot time in time namespace",
-            "timens_offsets": [moffset, boffset]);
+            proc_write(pid, b"timens_offsets", data.as_bytes())?;
+            info!("ctx": "set_boot_time", "msg": "set boot time in time namespace",
+                "timens_offsets": [moffset, boffset]);
+        }
     }
 
     // Disable Speculative Store Bypass mitigations
