@@ -32,6 +32,16 @@ mod test;
 mod util;
 use test::*;
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 #[derive(Debug)]
 struct TempDir {
     path: XPathBuf,
@@ -121,7 +131,7 @@ fn main() -> SydResult<ExitCode> {
     umask(Mode::from_bits_truncate(0o077));
 
     println!("# syd-test: Welcome to the Machine!");
-    println!("# usage: syd-test [-hlq] [<name-regex>|<number>|<number>..<number>]..");
+    println!("# usage: syd-test [-hlq] [<name-glob>|<number>|<number>..<number>]..");
 
     let args = std::env::args().skip(1).collect::<Vec<_>>();
     let mut args_is_empty = args.is_empty();
@@ -129,7 +139,7 @@ fn main() -> SydResult<ExitCode> {
 
     if !args_is_empty && matches!(args[0].as_str(), "-h" | "--help" | "-l" | "--list") {
         for (idx, (name, _)) in TESTS.iter().enumerate() {
-            #[allow(clippy::disallowed_methods)]
+            #[expect(clippy::disallowed_methods)]
             let name = name.strip_prefix("test_syd_").unwrap();
             let idx = idx + 1;
             println!("{idx:>3}: {name}");
@@ -158,7 +168,7 @@ fn main() -> SydResult<ExitCode> {
     // Create a temporary directory and enter it, failures are OK.
     // The directory is removed when the guard is dropped.
     let tmpdir = {
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let tmp = format!(
             "{}/syd_test_XXXXXX",
             env::var("SYD_TEST_TMPDIR").unwrap_or(".".to_string())
@@ -193,7 +203,7 @@ fn main() -> SydResult<ExitCode> {
 
     // Step 1: Handle the SYD_TEST environment variable.
     let mut test_env_arg = false;
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     if let Ok(env) = std::env::var("SYD_TEST") {
         if !env.is_empty() {
             test_env_arg = true;
@@ -204,7 +214,7 @@ fn main() -> SydResult<ExitCode> {
                 Arguments::Range(r) => test_indices.extend(r),
                 Arguments::Pattern(p) => {
                     for (idx, (name, _)) in TESTS.iter().enumerate() {
-                        #[allow(clippy::disallowed_methods)]
+                        #[expect(clippy::disallowed_methods)]
                         let name = name.strip_prefix("test_syd_").unwrap();
                         if inamematch(&p, name) {
                             test_indices.push(idx + 1);
@@ -231,7 +241,7 @@ fn main() -> SydResult<ExitCode> {
             Arguments::Range(r) => test_indices.extend(r),
             Arguments::Pattern(p) => {
                 for (idx, (name, _)) in TESTS.iter().enumerate() {
-                    #[allow(clippy::disallowed_methods)]
+                    #[expect(clippy::disallowed_methods)]
                     let name = name.strip_prefix("test_syd_").unwrap();
                     if inamematch(&p, name) {
                         test_indices.push(idx + 1);
@@ -251,10 +261,10 @@ fn main() -> SydResult<ExitCode> {
     }
 
     // Shuffle test indices with an optional seed.
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let (seed, seed_set) = match env::var("SYD_TEST_SEED") {
         Err(env::VarError::NotPresent) => {
-            let mut buf = vec![0u8; std::mem::size_of::<nix::libc::c_uint>()];
+            let mut buf = vec![0u8; size_of::<nix::libc::c_uint>()];
             let ret = unsafe {
                 nix::libc::getrandom(buf.as_mut_ptr().cast(), buf.len(), nix::libc::GRND_RANDOM)
             };
@@ -297,7 +307,12 @@ fn main() -> SydResult<ExitCode> {
         return Ok(ExitCode::from(0));
     }
     let ntest = test_indices.len();
-    println!("1..{ntest}");
+    let ptest = if env::var_os("SYD_TEST_REEXEC").is_none() {
+        ""
+    } else {
+        "#    "
+    };
+    println!("{ptest}1..{ntest}");
 
     let exp_test = env::var_os("SYD_TEST_EXPENSIVE").is_some();
     let mut fail_hard = 0;
@@ -315,8 +330,9 @@ fn main() -> SydResult<ExitCode> {
             continue;
         };
         idx += 1;
-        #[allow(clippy::disallowed_methods)]
+        #[expect(clippy::disallowed_methods)]
         let name = name.strip_prefix("test_syd_").unwrap();
+        env::set_var("SYD_TEST_NAME", name);
         if exp_test && !name.starts_with("exp_") {
             println!("# ok {idx} - {name} # SKIP not expensive, unset SYD_TEST_EXPENSIVE to run");
             skip += 1;
@@ -358,7 +374,6 @@ fn main() -> SydResult<ExitCode> {
         std::env::set_var("TMP", &cwd);
         std::env::set_var("TMPDIR", &cwd);
         let result = test();
-        #[allow(clippy::disallowed_methods)]
         chdir("..")?;
         rtest += 1;
         match result {
@@ -367,9 +382,9 @@ fn main() -> SydResult<ExitCode> {
                     fail_soft += 1;
                     soft_fails.push(name.to_string());
                     std::env::remove_var("SYD_TEST_SOFT_FAIL");
-                    println!("ok {idx} - {name} # TODO");
+                    println!("{ptest}ok {idx} - {name} # TODO");
                 } else {
-                    println!("ok {idx} - {name}");
+                    println!("{ptest}ok {idx} - {name}");
                 }
                 if tmpdir.as_ref().map(|t| t.keep).unwrap_or(false) {
                     let cwd = getcwd()
@@ -381,7 +396,7 @@ fn main() -> SydResult<ExitCode> {
                 }
             }
             Err(error) => {
-                println!("not ok {idx} - {name} - FAIL: {error}");
+                println!("{ptest}not ok {idx} - {name} - FAIL: {error}");
                 fail_hard += 1;
                 fail_names.push(name.to_string());
                 if fail_quick {
