@@ -21,6 +21,16 @@ use nix::{
 };
 use syd::{path::XPathBuf, proc::proc_environ_read, syslog::LogLevel, wordexp::WordExp};
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 syd::main! {
     syd::set_sigpipe_dfl()?;
 
@@ -63,7 +73,7 @@ syd::main! {
     let mut pfd = XPathBuf::from("/proc");
     pfd.push_pid(pid);
     pfd.push(b"environ");
-    #[allow(clippy::disallowed_methods)]
+    #[expect(clippy::disallowed_methods)]
     let pfd = open(&pfd, OFlag::O_RDONLY, Mode::empty())?;
 
     let environ = match proc_environ_read(pfd) {
