@@ -55,6 +55,7 @@ syd::main! {
             println!("If set is personality, print the list of allowed personalities.");
             println!("Available sets are:");
             println!("- cpu");
+            println!("- dead");
             println!("- deny");
             println!("- deprecated");
             println!("- ebpf");
@@ -104,6 +105,11 @@ syd::main! {
                 println!("{name}");
             }
         }
+        Some("dead") => {
+            for name in syd::config::DEAD_SYSCALLS {
+                println!("{name}");
+            }
+        }
         Some("deprecated") => {
             for name in syd::config::DEPRECATED_SYSCALLS {
                 println!("{name}");
