@@ -13,6 +13,16 @@ use std::{
     time::Instant,
 };
 
+// Set global allocator to mimalloc.
+#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
+#[global_allocator]
+static GLOBAL: mimalloc::MiMalloc = mimalloc::MiMalloc;
+
+// Set global allocator to tcmalloc if profiling is enabled.
+#[cfg(feature = "prof")]
+#[global_allocator]
+static GLOBAL: tcmalloc::TCMalloc = tcmalloc::TCMalloc;
+
 #[cfg(target_arch = "x86_64")]
 use tick_counter::x86_64_processor_id;
 // Conditional compilation for architecture-specific features
@@ -73,7 +83,7 @@ fn elapsed_tick(start_tick: &Instant) -> u64 {
 }
 
 // XXX: false positive.
-#[allow(clippy::needless_return)]
+#[expect(clippy::needless_return)]
 fn tick_info() -> (u64, f64) {
     #[cfg(any(target_arch = "aarch64", target_arch = "x86_64"))]
     {
