@@ -43,6 +43,7 @@ use crate::{
 
 #[derive(Clone, Copy, Debug, Eq, PartialEq)]
 enum InterruptSource {
+    Manual,
     Signal(SydSigSet),
     ProcessInvalid(Errno),
 }
@@ -54,6 +55,9 @@ impl Serialize for InterruptSource {
     {
         let mut map = serializer.serialize_map(Some(2))?;
         match self {
+            Self::Manual => {
+                map.serialize_entry("name", "manual")?;
+            }
             Self::Signal(set) => {
                 map.serialize_entry("name", "signal")?;
                 map.serialize_entry("set", set)?;
@@ -250,6 +254,11 @@ impl Interrupter {
         if interrupt.delete {
             return false;
         }
+        // Check if syd_mon requested manual interruption.
+        if interrupt.signal {
+            Self::interrupt(tgid, interrupt, InterruptSource::Manual);
+            return false;
+        }
 
         // Open proc_pid_status(5) if it's not open already.
         //
