@@ -397,6 +397,9 @@ impl IpcWorker {
         // Deny open and stat family with ENOSYS rather than KillProcess.
         // We need this because Rust allocator has side effects such as
         // opening /proc/sys/vm/overcommit_memory on some architectures.
+        //
+        // Note, we avoid this when profiling is enabled,
+        // as gperf requires it to write profiling data.
         for sysname in [
             "open",
             "openat",
@@ -408,7 +411,12 @@ impl IpcWorker {
         ] {
             match ScmpSyscall::from_name(sysname) {
                 Ok(syscall) => {
-                    ctx.add_rule(ScmpAction::Errno(Errno::ENOSYS as i32), syscall)?;
+                    let action = if !cfg!(feature = "prof") {
+                        ScmpAction::Errno(Errno::ENOSYS as i32)
+                    } else {
+                        ScmpAction::Allow
+                    };
+                    ctx.add_rule(action, syscall)?;
                 }
                 Err(_) => {
                     info!("ctx": "confine", "op": "allow_ipc_syscall",
